==> cpupowerd-0.2.0.tar.gz: cpupowerd-0.2.0/src/cpu_linux.c <==
/************************************************************************/
/* Copyright (C) 2008 Markus Strobl <markus.strobl@gmx.at>              */
/*                                                                      */
/* cpupowerd is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* cpupowerd is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with GNU Emacs; see the file COPYING.  If not, write to        */
/* the Free Software Foundation, Inc., 59 Temple Place - Suite 330,     */
/* Boston, MA 02111-1307, USA.                                          */
/************************************************************************/
/* $Id: cpu_linux.c 309 2008-11-02 13:24:50Z markus $                   */
/************************************************************************/


/* Includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "cpupowerd.h"
#include "libcpupowerd.h"
#include "cpu_linux.h"


/* Defines */
#define CPUFREQ_PATH 							"/sys/devices/system/cpu/"
#define CPUFREQ_DIR 							"cpufreq/"
#define CPUFREQ_AFFECTES_CPUS 					"affected_cpus"
#define CPUFREQ_SCALING_AVAILABLE_FREQUENCIES 	"scaling_available_frequencies"
#define CPUFREQ_SCALING_AVAILABLE_GOVERNORS		"scaling_available_governors"
#define CPUFREQ_SET_FREQUENCY 					"scaling_setspeed"
#define CPUFREQ_CURRENT_FREQUENCY 				"scaling_cur_freq"
#define CPUFREQ_GOVERNOR 						"scaling_governor"
#define PROC_STATUS_FILE						"/proc/stat"
#define PROC_LAPTOP_MODE_FILE					"/proc/sys/vm/laptop_mode"


/* Prototypes */
int cpu_linux_tini(void);
int cpu_linux_get_load(int cpuid, unsigned long long *total, unsigned long long *usage, float *load);
int cpu_linux_get_laptop_mode(int *laptop_mode);
int cpu_linux_get_mastercpuids(int *cpuids, int **mastercpuids, int *quantity);
int cpu_linux_get_affectedcpuids(int mastercpuid, int **affected_cpuids, int *quantity);
int cpu_linux_get_frequencies(int mastercpuid, int **frequencies, int *quantity);
int cpu_linux_get_frequency(int mastercpuid, int *frequency);
int cpu_linux_set_frequency(int mastercpuid, int frequency);
int cpu_linux_get_governor(int mastercpuid, char *governor, int size);
int cpu_linux_set_governor(int mastercpuid, char *governor);
static int cpu_linux_is_governor(int mastercpuid, char *governor);


/* Functions */
/* Init */
int cpu_linux_init(int (**ptr_cpu_get_load) (int, unsigned long long *, unsigned long long *, float *),
				   int (**ptr_cpu_get_laptop_mode) (int *),
				   int (**ptr_cpu_get_mastercpuids) (int*, int **, int*),
				   int (**ptr_cpu_get_affectedcpuids) (int, int **, int *),
				   int (**ptr_cpu_get_frequencies) (int, int **, int *),
				   int (**ptr_cpu_get_frequency) (int, int *),
				   int (**ptr_cpu_set_frequency) (int, int),
				   int (**ptr_cpu_get_governor) (int, char *, int),
				   int (**ptr_cpu_set_governor) (int, char *),
				   int (**ptr_cpu_tini) (void))
{
	*ptr_cpu_get_load = cpu_linux_get_load;
	*ptr_cpu_get_laptop_mode = cpu_linux_get_laptop_mode;
	*ptr_cpu_get_mastercpuids = cpu_linux_get_mastercpuids;
	*ptr_cpu_get_affectedcpuids = cpu_linux_get_affectedcpuids;
	*ptr_cpu_get_frequencies = cpu_linux_get_frequencies;
	*ptr_cpu_get_frequency = cpu_linux_get_frequency;
	*ptr_cpu_set_frequency = cpu_linux_set_frequency;
	*ptr_cpu_get_governor = cpu_linux_get_governor;
	*ptr_cpu_set_governor = cpu_linux_set_governor;
	*ptr_cpu_tini = cpu_linux_tini;

	return OK;
}


/* Clean up */
int cpu_linux_tini(void)
{
	return OK;
}


/* Get load for mastercpuid */
/* Format: cpu<id> <user> <nice> <system> <idle> <iowait> <irq> <softirq> */
int cpu_linux_get_load(int cpuid, unsigned long long *total, unsigned long long *usage, float *load)
{	
	int size;
	unsigned long long stat_user;
	unsigned long long stat_system;
	unsigned long long stat_nice;
	unsigned long long stat_idle;
	unsigned long long stat_iowait;
	unsigned long long stat_irq;
	unsigned long long stat_softirq;
	unsigned long long total_current;
	unsigned long long usage_current;
	char *buffer;
	char *ptr_tmp;
	char search[16];

	if (libcpupowerd_read_file_dynamic(PROC_STATUS_FILE, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	snprintf(search, sizeof(search), "cpu%d ", cpuid);
	if ((ptr_tmp = strstr(buffer, search)) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "%s not found in %s!\n", search, PROC_STATUS_FILE);
		return NOTOK;
	}
	ptr_tmp += strlen(search);

	if (sscanf(ptr_tmp, "%llu %llu %llu %llu %llu %llu %llu", &stat_user, &stat_nice, &stat_system, &stat_idle, &stat_iowait, &stat_irq, &stat_softirq) != 7)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't read cpu stats for cpuid %d!\n", cpuid);
		return NOTOK;
	}

	total_current = stat_user + stat_nice + stat_system + stat_irq + stat_softirq + stat_iowait + stat_idle;
	usage_current = stat_user + stat_nice + stat_system + stat_irq + stat_softirq;

	*load = (float)(usage_current - *usage) / (float)(total_current - *total);

	*total = total_current;
	*usage = usage_current;

	free(buffer);

	return OK;
}


/* Get laptop mode */
int cpu_linux_get_laptop_mode(int *laptop_mode)
{
	char buffer[8];

	if (libcpupowerd_read_file(PROC_LAPTOP_MODE_FILE, 0, 0, buffer, (int)sizeof(buffer)) != LIBCPUPOWERD_OK)
	{
		*laptop_mode = 0; /* Default laptop_mode is zero */
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_INFO, "Laptop mode can't be detected, using laptop mode 0!\n");
	}
	else
		*laptop_mode = atoi(buffer);

	return OK;
}


/* Get cpu mastercpuids */
int cpu_linux_get_mastercpuids(int *cpuids, int **mastercpuids, int *quantity)
{
	int i, j, k;
	int found;
	int quantity_cpuids;
	int quantity_affectedcpuids;
	int *affectedcpuids;
	
	quantity_cpuids = 0;
	while(cpuids[quantity_cpuids] >= 0)
		quantity_cpuids++;

	/* Allocate mastercpuids array with the quantity of cpuids, this is more than enough */
	*quantity = 0;
	if ((*mastercpuids = malloc((quantity_cpuids + 1) * sizeof(int))) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't allocate memory for mastercpuids!\n");
		return NOTOK;
	}
	(*mastercpuids)[*quantity] = -1;

	i = 0;
	while (cpuids[i] >= 0)
	{
		/* Get affected cpuids for cpuid */
		if (cpu_linux_get_affectedcpuids(cpuids[i], &affectedcpuids, &quantity_affectedcpuids) != OK)
			return NOTOK;
		
		/* Is one of the affected cpuids already in the mastercpuids array? */
		j = 0;
		found = 0;
		while ((*mastercpuids)[j] >= 0)
		{
			k = 0;	
			while (affectedcpuids[k] >= 0)
			{
				if ((*mastercpuids)[j] == affectedcpuids[k])
				{
					found = 1;
					break;
				}

				k++;
			}
			if (found == 1)
				break;

			j++;
		}

		/* No affected cpuid is in the mastercpuids array available -> add current cpuid */
		if (found != 1)
		{
			(*mastercpuids)[*quantity] = cpuids[i];
			(*quantity)++;
		}

		free(affectedcpuids);
		i++;		
	}
	(*mastercpuids)[*quantity] = -1;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No master cpuids found!\n");
		return NOTOK;
	}

	return OK;
}


/* Get affected cpuids */
int cpu_linux_get_affectedcpuids(int mastercpuid, int **affectedcpuids, int *quantity)
{
	int i;
	char file[128];
	char buffer[128];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_AFFECTES_CPUS);
	if (libcpupowerd_read_file(file, 0, 0, buffer, (int)sizeof(buffer)) != LIBCPUPOWERD_OK)
		return NOTOK;
	
	if (libcpupowerd_get_integers_from_string(buffer, ' ', affectedcpuids, quantity) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No affected cpuids found!\n");
		return NOTOK;
	}

	/* Check, if mastercpuid is in affected cpuids array */
	i = 0;
	while ((*affectedcpuids)[i] >= 0)
	{
		if ((*affectedcpuids)[i] == mastercpuid)
			return OK;

		i++;
	}

	libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Mastercpuid %d isn't in affected cpuids array!\n", mastercpuid);

	return NOTOK;
}


/* Get cpu frequencies */
int cpu_linux_get_frequencies(int mastercpuid, int **frequencies, int *quantity)
{
	int i;
	char file[128];
	char buffer[128];
	
	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SCALING_AVAILABLE_FREQUENCIES);
	if (libcpupowerd_read_file(file, 0, 0, buffer, (int)sizeof(buffer)) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (libcpupowerd_get_integers_from_string(buffer, ' ', frequencies, quantity) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No frequencies found!\n");
		return NOTOK;
	}

	/* Change from kHz to MHz */
	i = 0;
	while((*frequencies)[i] >= 0)
	{
		(*frequencies)[i] = (*frequencies)[i]/1000;
		i++;
	}

	return OK;
}


/* Get frequency */
int cpu_linux_get_frequency(int mastercpuid, int *frequency)
{
	char file[128];
	char buffer[16];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_CURRENT_FREQUENCY);
	if (libcpupowerd_read_file(file, 0, 0, buffer, (int)sizeof(buffer)) != LIBCPUPOWERD_OK)
		return NOTOK;

	*frequency = atoi(buffer)/1000; /* Change from kHz to MHz */

	return OK;
}


/* Set frequency */
int cpu_linux_set_frequency(int mastercpuid, int frequency)
{
	int found;
	int counter;
	int quantity;
	int *frequencies;
	char file[128];
	char buffer[16];

	if (cpu_linux_get_frequencies(mastercpuid, &frequencies, &quantity) != OK)
		return NOTOK;

	found = 0;
	counter = 0;
	while (frequencies[counter] >= 0)
	{
		if (frequencies[counter] == frequency)
		{
			found = 1;
			break;
		}

		counter++;
	}
	free(frequencies);

	if (found == 0)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Frequency %d MHz not found in available frequencies!\n", frequency);
		return NOTOK;
	}

	frequency = frequency * 1000; /* Change from MHz to kHz */
	snprintf(buffer, sizeof(buffer), "%d", frequency);
	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SET_FREQUENCY);

	if (libcpupowerd_write_file(file, 0, buffer) != LIBCPUPOWERD_OK)
		return NOTOK;

	return OK;
}


/* Get governor */
int cpu_linux_get_governor(int mastercpuid, char *governor, int size)
{
	char file[128];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_GOVERNOR);
	if (libcpupowerd_read_file(file, 0, 0, governor, size) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (governor[(int)strlen(governor) - 1] == '\n')
		governor[(int)strlen(governor) - 1] = '\0';

	return OK;
}


/* Set governor */
int cpu_linux_set_governor(int mastercpuid, char *governor)
{
	char file[128];
	char governor_tmp[32];

	/* Compare with current governor */
	if (cpu_linux_get_governor(mastercpuid, governor_tmp, (int)sizeof(governor_tmp)) != OK)
		return NOTOK;

	if (strcmp(governor, governor_tmp) == 0)
		return OK;

	/* Is governor available */
	if (cpu_linux_is_governor(mastercpuid, governor) != OK)
		return NOTOK;

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_GOVERNOR);
	if (libcpupowerd_write_file(file, 0, governor) != LIBCPUPOWERD_OK)
		return NOTOK;

	return OK;
}


/* Check if governor is available */
static int cpu_linux_is_governor(int mastercpuid, char *governor)
{
	int size;
	char *buffer;
	char file[128];
	char search_str[32];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SCALING_AVAILABLE_GOVERNORS);
	if (libcpupowerd_read_file_dynamic(file, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	snprintf(search_str, sizeof(search_str), "%s", governor);
	if (strstr(buffer, search_str) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Governor %s isn't available!\n", governor);
		return NOTOK;
	}
		
	free(buffer);

	return OK;
}

==> cpupowerd-0.2.1.tar.gz: cpupowerd-0.2.1/src/cpu_linux.c <==
/************************************************************************/
/* Copyright (C) 2008-2009 Markus Strobl <markus.strobl@gmx.at>         */
/*                                                                      */
/* cpupowerd is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* cpupowerd is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with GNU Emacs; see the file COPYING.  If not, write to        */
/* the Free Software Foundation, Inc., 59 Temple Place - Suite 330,     */
/* Boston, MA 02111-1307, USA.                                          */
/************************************************************************/
/* $Id: cpu_linux.c 369 2009-01-01 13:05:07Z markus $                   */
/************************************************************************/


/* Includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "cpupowerd.h"
#include "libcpupowerd.h"
#include "cpu_linux.h"


/* Defines */
#define CPUFREQ_PATH 							"/sys/devices/system/cpu/"
#define CPUFREQ_DIR 							"cpufreq/"
#define CPUFREQ_AFFECTES_CPUS 					"affected_cpus"
#define CPUFREQ_SCALING_AVAILABLE_FREQUENCIES 	"scaling_available_frequencies"
#define CPUFREQ_SCALING_AVAILABLE_GOVERNORS		"scaling_available_governors"
#define CPUFREQ_SET_FREQUENCY 					"scaling_setspeed"
#define CPUFREQ_CURRENT_FREQUENCY 				"scaling_cur_freq"
#define CPUFREQ_GOVERNOR 						"scaling_governor"
#define PROC_STATUS_FILE						"/proc/stat"
#define PROC_LAPTOP_MODE_FILE					"/proc/sys/vm/laptop_mode"


/* Prototypes */
int cpu_linux_tini(void);
int cpu_linux_get_load(int cpuid, unsigned long long *total, unsigned long long *usage, float *load);
int cpu_linux_get_laptop_mode(int *laptop_mode);
int cpu_linux_get_mastercpuids(int *cpuids, int **mastercpuids, int *quantity);
int cpu_linux_get_affectedcpuids(int mastercpuid, int **affected_cpuids, int *quantity);
int cpu_linux_get_frequencies(int mastercpuid, int **frequencies, int *quantity);
int cpu_linux_get_frequency(int mastercpuid, int *frequency);
int cpu_linux_set_frequency(int mastercpuid, int frequency);
int cpu_linux_get_governor(int mastercpuid, char *governor, int size);
int cpu_linux_set_governor(int mastercpuid, char *governor);
static int cpu_linux_is_governor(int mastercpuid, char *governor);


/* Functions */
/* Init */
int cpu_linux_init(int (**ptr_cpu_get_load) (int, unsigned long long *, unsigned long long *, float *),
				   int (**ptr_cpu_get_laptop_mode) (int *),
				   int (**ptr_cpu_get_mastercpuids) (int*, int **, int*),
				   int (**ptr_cpu_get_affectedcpuids) (int, int **, int *),
				   int (**ptr_cpu_get_frequencies) (int, int **, int *),
				   int (**ptr_cpu_get_frequency) (int, int *),
				   int (**ptr_cpu_set_frequency) (int, int),
				   int (**ptr_cpu_get_governor) (int, char *, int),
				   int (**ptr_cpu_set_governor) (int, char *),
				   int (**ptr_cpu_tini) (void))
{
	*ptr_cpu_get_load = cpu_linux_get_load;
	*ptr_cpu_get_laptop_mode = cpu_linux_get_laptop_mode;
	*ptr_cpu_get_mastercpuids = cpu_linux_get_mastercpuids;
	*ptr_cpu_get_affectedcpuids = cpu_linux_get_affectedcpuids;
	*ptr_cpu_get_frequencies = cpu_linux_get_frequencies;
	*ptr_cpu_get_frequency = cpu_linux_get_frequency;
	*ptr_cpu_set_frequency = cpu_linux_set_frequency;
	*ptr_cpu_get_governor = cpu_linux_get_governor;
	*ptr_cpu_set_governor = cpu_linux_set_governor;
	*ptr_cpu_tini = cpu_linux_tini;

	return OK;
}


/* Clean up */
int cpu_linux_tini(void)
{
	return OK;
}


/* Get load for mastercpuid */
/* Format: cpu<id> <user> <nice> <system> <idle> <iowait> <irq> <softirq> */
int cpu_linux_get_load(int cpuid, unsigned long long *total, unsigned long long *usage, float *load)
{	
	int size;
	unsigned long long stat_user;
	unsigned long long stat_system;
	unsigned long long stat_nice;
	unsigned long long stat_idle;
	unsigned long long stat_iowait;
	unsigned long long stat_irq;
	unsigned long long stat_softirq;
	unsigned long long total_current;
	unsigned long long usage_current;
	char *buffer;
	char *ptr_tmp;
	char search[16];

	if (libcpupowerd_read_file_dynamic(PROC_STATUS_FILE, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	snprintf(search, sizeof(search), "cpu%d ", cpuid);
	if ((ptr_tmp = strstr(buffer, search)) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "%s not found in %s!\n", search, PROC_STATUS_FILE);
		return NOTOK;
	}
	ptr_tmp += strlen(search);

	if (sscanf(ptr_tmp, "%llu %llu %llu %llu %llu %llu %llu", &stat_user, &stat_nice, &stat_system, &stat_idle, &stat_iowait, &stat_irq, &stat_softirq) != 7)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't read cpu stats for cpuid %d!\n", cpuid);
		return NOTOK;
	}

	total_current = stat_user + stat_nice + stat_system + stat_irq + stat_softirq + stat_iowait + stat_idle;
	usage_current = stat_user + stat_nice + stat_system + stat_irq + stat_softirq;

	*load = (float)(usage_current - *usage) / (float)(total_current - *total);

	*total = total_current;
	*usage = usage_current;

	free(buffer);

	return OK;
}


/* Get laptop mode */
int cpu_linux_get_laptop_mode(int *laptop_mode)
{
	int readed_bytes;
	char buffer[8];

	if (libcpupowerd_read_file(PROC_LAPTOP_MODE_FILE, 0, 0, (void *)buffer, (int)sizeof(buffer), &readed_bytes) != LIBCPUPOWERD_OK)
	{
		*laptop_mode = 0; /* Default laptop_mode is zero */
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_INFO, "Laptop mode can't be detected, using laptop mode 0!\n");
	}
	else
	{
		buffer[readed_bytes] = '\0';
		*laptop_mode = atoi(buffer);
	}

	return OK;
}


/* Get cpu mastercpuids */
int cpu_linux_get_mastercpuids(int *cpuids, int **mastercpuids, int *quantity)
{
	int i, j, k;
	int found;
	int quantity_cpuids;
	int quantity_affectedcpuids;
	int *affectedcpuids;
	
	quantity_cpuids = 0;
	while(cpuids[quantity_cpuids] >= 0)
		quantity_cpuids++;

	/* Allocate mastercpuids array with the quantity of cpuids, this is more than enough */
	*quantity = 0;
	if ((*mastercpuids = malloc((quantity_cpuids + 1) * sizeof(int))) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't allocate memory for mastercpuids!\n");
		return NOTOK;
	}
	(*mastercpuids)[*quantity] = -1;

	i = 0;
	while (cpuids[i] >= 0)
	{
		/* Get affected cpuids for cpuid */
		if (cpu_linux_get_affectedcpuids(cpuids[i], &affectedcpuids, &quantity_affectedcpuids) != OK)
			return NOTOK;
		
		/* Is one of the affected cpuids already in the mastercpuids array? */
		j = 0;
		found = 0;
		while ((*mastercpuids)[j] >= 0)
		{
			k = 0;	
			while (affectedcpuids[k] >= 0)
			{
				if ((*mastercpuids)[j] == affectedcpuids[k])
				{
					found = 1;
					break;
				}

				k++;
			}
			if (found == 1)
				break;

			j++;
		}

		/* No affected cpuid is in the mastercpuids array available -> add current cpuid */
		if (found != 1)
		{
			(*mastercpuids)[*quantity] = cpuids[i];
			(*quantity)++;
		}

		free(affectedcpuids);
		i++;		
	}
	(*mastercpuids)[*quantity] = -1;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No master cpuids found!\n");
		return NOTOK;
	}

	return OK;
}


/* Get affected cpuids */
int cpu_linux_get_affectedcpuids(int mastercpuid, int **affectedcpuids, int *quantity)
{
	int i;
	int readed_bytes;
	char file[128];
	char buffer[128];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_AFFECTES_CPUS);
	if (libcpupowerd_read_file(file, 0, 0, (void *)buffer, (int)sizeof(buffer), &readed_bytes) != LIBCPUPOWERD_OK)
		return NOTOK;

	buffer[readed_bytes] = '\0';
	
	if (libcpupowerd_get_integers_from_string(buffer, "%d", ' ', affectedcpuids, quantity) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No affected cpuids found!\n");
		return NOTOK;
	}

	/* Check, if mastercpuid is in affected cpuids array */
	i = 0;
	while ((*affectedcpuids)[i] >= 0)
	{
		if ((*affectedcpuids)[i] == mastercpuid)
			return OK;

		i++;
	}

	libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Mastercpuid %d isn't in affected cpuids array!\n", mastercpuid);

	return NOTOK;
}


/* Get cpu frequencies */
int cpu_linux_get_frequencies(int mastercpuid, int **frequencies, int *quantity)
{
	int i;
	int readed_bytes;
	char file[128];
	char buffer[128];
	
	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SCALING_AVAILABLE_FREQUENCIES);
	if (libcpupowerd_read_file(file, 0, 0, (void *)buffer, (int)sizeof(buffer), &readed_bytes) != LIBCPUPOWERD_OK)
		return NOTOK;

	buffer[readed_bytes] = '\0';

	if (libcpupowerd_get_integers_from_string(buffer, "%d", ' ', frequencies, quantity) != LIBCPUPOWERD_OK)
		return NOTOK;

	if (*quantity < 1)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "No frequencies found!\n");
		return NOTOK;
	}

	/* Change from kHz to MHz */
	i = 0;
	while((*frequencies)[i] >= 0)
	{
		(*frequencies)[i] = (*frequencies)[i]/1000;
		i++;
	}

	return OK;
}


/* Get frequency */
int cpu_linux_get_frequency(int mastercpuid, int *frequency)
{
	int readed_bytes;
	char file[128];
	char buffer[16];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_CURRENT_FREQUENCY);
	if (libcpupowerd_read_file(file, 0, 0, (void *)buffer, (int)sizeof(buffer), &readed_bytes) != LIBCPUPOWERD_OK)
		return NOTOK;

	buffer[readed_bytes] = '\0';

	*frequency = atoi(buffer)/1000; /* Change from kHz to MHz */

	return OK;
}


/* Set frequency */
int cpu_linux_set_frequency(int mastercpuid, int frequency)
{
	int found;
	int counter;
	int quantity;
	int *frequencies;
	char file[128];
	char buffer[16];

	if (cpu_linux_get_frequencies(mastercpuid, &frequencies, &quantity) != OK)
		return NOTOK;

	found = 0;
	counter = 0;
	while (frequencies[counter] >= 0)
	{
		if (frequencies[counter] == frequency)
		{
			found = 1;
			break;
		}

		counter++;
	}
	free(frequencies);

	if (found == 0)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Frequency %d MHz not found in available frequencies!\n", frequency);
		return NOTOK;
	}

	frequency = frequency * 1000; /* Change from MHz to kHz */
	snprintf(buffer, sizeof(buffer), "%d", frequency);
	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SET_FREQUENCY);

	if (libcpupowerd_write_file(file, 0, strlen(buffer), (void *)buffer) != LIBCPUPOWERD_OK)
		return NOTOK;

	return OK;
}


/* Get governor */
int cpu_linux_get_governor(int mastercpuid, char *governor, int size)
{
	int readed_bytes;
	char file[128];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_GOVERNOR);
	if (libcpupowerd_read_file(file, 0, 0, (void *)governor, size, &readed_bytes) != LIBCPUPOWERD_OK)
		return NOTOK;

	governor[readed_bytes] = '\0';

	if (governor[(int)strlen(governor) - 1] == '\n')
		governor[(int)strlen(governor) - 1] = '\0';

	return OK;
}


/* Set governor */
int cpu_linux_set_governor(int mastercpuid, char *governor)
{
	char file[128];
	char governor_tmp[32];

	/* Compare with current governor */
	if (cpu_linux_get_governor(mastercpuid, governor_tmp, (int)sizeof(governor_tmp)) != OK)
		return NOTOK;

	if (strcmp(governor, governor_tmp) == 0)
		return OK;

	/* Is governor available */
	if (cpu_linux_is_governor(mastercpuid, governor) != OK)
		return NOTOK;

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_GOVERNOR);
	if (libcpupowerd_write_file(file, 0, strlen(governor), (void *)governor) != LIBCPUPOWERD_OK)
		return NOTOK;

	return OK;
}


/* Check if governor is available */
static int cpu_linux_is_governor(int mastercpuid, char *governor)
{
	int size;
	char *buffer;
	char file[128];
	char search_str[32];

	snprintf(file, sizeof(file), "%scpu%d/%s%s", CPUFREQ_PATH, mastercpuid, CPUFREQ_DIR, CPUFREQ_SCALING_AVAILABLE_GOVERNORS);
	if (libcpupowerd_read_file_dynamic(file, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	snprintf(search_str, sizeof(search_str), "%s", governor);
	if (strstr(buffer, search_str) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Governor %s isn't available!\n", governor);
		return NOTOK;
	}
		
	free(buffer);

	return OK;
}
